% Input:
%   input = 'data' and data = #features x #samples data matrix
%   input = 'cov' and data = #features x #features covariance matrix
%   rho = regularization parameter
%   low_memory = true, remove mean in same data matrix and returns inverse as structure
%   low_memory = false, remove mean by using repmat and return inverse as matrix
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [OmegaRic OmegaTik] = learn_riccati_ggm(input,data,rho,low_memory);

if ~exist('low_memory','var')
    low_memory = false;
end
if strcmp(input,'data')
    [N p] = size(data);
    mu = mean(data,2);
    if low_memory
        for t = 1:p
            data(:,t) = data(:,t)-mu;
        end
        [U E] = svd(data,'econ');
    else
        [U E] = svd(data-repmat(mu,1,p),'econ');
    end
    d = diag(E).^2/p;
elseif strcmp(input,'cov')
    N = size(data,1);
    [U E] = svd(data);
    d = diag(E);
else
    error('Dont know what to do!\n');
end
dRic = sqrt(4/rho + d.^2/rho^2) - d/rho - 2/sqrt(rho);
if low_memory
    OmegaRic.U = U;
    OmegaRic.fU = dRic/2;
    OmegaRic.fI = 1/sqrt(rho);
else
    OmegaRic = U*diag(dRic/2)*U.' + 1/sqrt(rho)*eye(N);
end
if nargout >= 2
    dTik = -d./(rho*(d+rho));
    if low_memory
        OmegaTik.U = U;
        OmegaTik.fU = dTik;
        OmegaTik.fI = 1/rho;
    else
        OmegaTik = U*diag(dTik)*U.' + 1/rho*eye(N);
    end
end
