clear all
clc

N = 10; % variables
p = 50; % samples
T = 10; % iterations
rho = 0.1;
rho_diag = 0;
tau = 0.01;

fprintf('*** Locally constant (NIPS''09) ***\n\n');
S = cov(randn(p,N));
P = 1; % assume data is 1-dimensional
D = zeros(N-1,N); % derivative operator: D(i,:) have nonzeros on neighbors
for n = 1:N-1
    D(n,n) = 1;
    D(n,n+1) = -1;
end
X = learn_smooth_ggm(S, rho, rho_diag, tau, P, D, T);
disp(X);

fprintf('*** Multitask (ICML''10 and JMLR''12 submission) ***\n\n');
K = 5; % tasks
for k = 1:K
    S(:,:,k) = cov(randn(p,N)); % covariance for task k
end
NS = repmat(N,1,K); % number of samples for each task
group_norm = 2; % L_{1,2}
X = learn_multitask_ggm(S, NS, rho, rho_diag, group_norm, T);
disp('L_{1,2} regularizer, task 1')
disp(X(:,:,1));

group_norm = inf; % L_{1,inf}
X = learn_multitask_ggm(S, NS, rho, rho_diag, group_norm, T);
disp('L_{1,inf} multitask, task 1')
disp(X(:,:,1));

fprintf('*** Variable selection (AISTATS''12) ***\n\n');
S = cov(randn(p,N));
group_norm = 2; % L_{1,2}
X = learn_featsel_ggm(S, rho, rho_diag, tau, group_norm, T);
disp('L_{1,2} regularizer')
disp(X);

group_norm = inf; % L_{1,inf}
X = learn_featsel_ggm(S, rho, rho_diag, tau, group_norm, T);
disp('L_{1,inf} regularizer')
disp(X);

fprintf('*** Low-rank (UAI''13) ***\n\n');
Y = randn(100,5); % 100 features, 5 samples
% 'data' and low_memory = true is recommended for large matrices
low_memory = true;
[OmRic OmTik] = learn_riccati_ggm('data',Y,0.1,low_memory);
disp('You could recover the Riccati full matrix as:');
disp('OmRic.U*diag(OmRic.fU)*OmRic.U.''+OmRic.fI*eye(100)');
disp('You could recover the Tikhonov full matrix as:');
disp('OmTik.U*diag(OmTik.fU)*OmTik.U.''+OmTik.fI*eye(100)');
